
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

public class LoggerProxy<T> implements InvocationHandler {

    private static final int MAX_PARAM_LENGTH = 500;

    private static Logger log  = LoggerFactory.getLogger( LoggerProxy.class);

    private T obj;

    private Class<T> clazz;


    public LoggerProxy( T obj, Class<T> clazz ){
        this.obj = obj;
        this.clazz = clazz;
    }

    public Object invoke(Object proxy, Method method, Object[] args ) throws Throwable{
        try{
            Object result = method.invoke( obj, args );
            log.info( "Invoked " + methodToString( method, args ) + ":" + objectToString( result ) );
            return result;
        }catch( InvocationTargetException e ){
            log.warn( "Error invoking " + methodToString( method, args ) );
            throw e.getTargetException();
        }catch( Exception e ){
            log.warn( "Error invoking " + methodToString( method, args ) );
            throw e;
        }
    }

    private String methodToString( Method method, Object[] args ){
        return String.format( "%s.%s(%s)", clazz.getSimpleName(), method.getName(), argsToString( args ) );
    }

    private String argsToString( Object[] args ){
        StringBuilder result = new StringBuilder();
        if( args != null ){
            for( Object arg : args ){
                if( result.length() > 0 )
                    result.append( "," );
                if( arg == null )
                    result.append( "null" );
                else{
                    result.append( "(" ).append( arg.getClass().getSimpleName() ).append( ")" );
                    result.append( "\"" ).append( objectToString( arg ) ).append( "\"" );
                }
            }
        }
        return result.toString();
    }

    private String objectToString( Object obj ){
        if( obj == null )
            return "null";


        String value = obj.toString();
        if( value.length() > MAX_PARAM_LENGTH )
            value = value.substring( 0, MAX_PARAM_LENGTH - 6 ) + "[...]";
        return value;
    }

    @SuppressWarnings( "unchecked" )
    public static <T> T wrap( T obj, Class<T> clazz ){
        return (T) Proxy.newProxyInstance(
                obj.getClass().getClassLoader(),
                obj.getClass().getInterfaces(),
                new LoggerProxy( obj, clazz )
        );
    }
}